<?php
// $Id: ontology_navigation_relationship_demo.inc,v 1.3 2006/01/29 16:09:42 dman Exp $

/** 
 * @file demo_relationship_structure.inc
 *
 * This is a run-once setup file that creates a nominal heirachical structure
 * using only the relationship links to define navigation between nodes.
 *
 * Include the file, and call the relationship_navigation_template() function
 * to populate the database with a dozen or so pages.
 *
 * author dman.
 */
function relationship_navigation_demo() {

  $num_parts = 2;
  $num_chapters = 3;
  $num_pages = 3;

  $link_on = NIDS; // relationships can be established between either paths
  // $link_on = PATHS; // or ids. Currently I'm finding it tricky to clean up if I use paths
  
  $edit = new StdClass();
  $edit->status = 1;
  $edit->promote = 0;
  $edit->path = 'template';
 // $edit->taxonomy = array();
  $edit->body = "Just another demo page";
  $edit->type = 'story';
  $edit->format = 3;

  // remember all things we make
  $node_handles = array();
  // this is a running pointer
  $previous = '';
  

  $path_prefix="demo/";

  // front page
  $edit->title = "Home Page";
  $edit->path = $path_prefix . "home";

  drupal_set_message('Creating '.$edit->title);
  $node_handles['home'] = node_save($edit);
  $previous = $edit->path;
  $node_handles[$edit->path] = node_save($edit);

  $chapter = 0;
  for($part = 1;$part <= $num_parts; $part++){
    $p_prefix = "Part $part";

    // parts one and two
    $edit->title = $p_prefix;
    $edit->path = make_path($edit->title); 

    drupal_set_message('Creating '.$edit->title);
    $p_label = $edit->path;
    $p_nid = $node_handles[$edit->path] = node_save($edit);

    if($link_on == NIDS){
      relationship_set($p_nid, PREDICATE_IS_CONTAINED_BY, $node_handles['home']);
      relationship_set($p_nid, PREDICATE_FOLLOWS, $node_handles[$previous]);
    } else {
      // I'll use aliases everywhere for the linking, instead of ids
      //    relationship_set($p_label, PREDICATE_IS_CONTAINED_BY, $node_handles['home']);
      //    relationship_set($p_label, PREDICATE_FOLLOWS, $previous);
    }

    $previous = $edit->path;
    
    for($c_loop = 1 ; $c_loop <= $num_chapters ; $c_loop ++){
      $chapter ++;
      $c_prefix = "Chapter $chapter";
      
      $edit->title = $p_prefix . ', ' . $c_prefix;
      $edit->path = make_path($edit->title);
      $c_label = $edit->path;
      $c_nid = $node_handles[$edit->path] = node_save($edit);

      if($link_on == NIDS){
        // chapters are contained by the current part
        relationship_set($c_nid, PREDICATE_IS_CONTAINED_BY, $node_handles[$p_label]);
        // everything come after whatever is previous
        relationship_set($c_nid, PREDICATE_FOLLOWS, $node_handles[$previous]);
      } else {
        relationship_set($c_label, PREDICATE_IS_CONTAINED_BY, $p_label);
        relationship_set($c_label, PREDICATE_FOLLOWS, $previous);
      }

      $previous = $edit->path;
    }
    
  }

  // Now associate this with a category for easy management
  // couldn't do this at create time as we needed a nid first.
  
  $terms = taxonomy_get_term_by_name('demo');
  if(! $terms ){
    $demo_term = array(
      'name'     => 'demo',
      'description' => 'auto-generated test content',
      'vid'      => 1, // hope this doesn't make a mess
      );
     taxonomy_save_term($demo_term);
     $terms = taxonomy_get_term_by_name('demo') ;
  }


  $out .= "";
  foreach($node_handles as $name => $nid){
    $out .= "<li>".l($name,'node/'.$nid)." #$nid</li>";
    $term_a = array( $terms[0]->tid );
    if($terms && $nid){
      taxonomy_node_save($nid, $term_a );
    }
  }

  // drupal_set_message(print_r($node_handles,1));
  return $out;
}

function make_path($title) {
  return "demo/" . preg_replace('/[^\w]+/','_',$title);
}

function relationship_navigation_demo_cleanup() {
  $tids = taxonomy_get_term_by_name('demo');
  if(is_array($tids)){
    $tax_term = array_shift($tids);
  }
  if($tax_term){
    $tid = $tax_term->tid;
  }
  
  if( $tid ){ 
    debug("Taxonomy id for 'demo' is $tid");
    $result = taxonomy_select_nodes(array($tid));

    while ($node = db_fetch_object($result)) {
      node_delete(array('nid' => $node->nid, 'confirm' => TRUE) );
    }

    // taxonomy_del_term($tid);
    drupal_set_message("Deleted all 'navigation demo' nodes.");
  } else {
    drupal_set_message("Failed to find the nodes labeled as 'demo' content.");
  }
}

?>